from pydantic import BaseModel, AnyUrl, Field

from apiiif.resource_properties import (
    LanguageString,
    LabelValue,
    Provider,
    Thumbnail,
    Homepage,
    PartOf,
    Image,
    Choice,
    ImageService,
    ExternalAuthService,
    BaseID,
)


class IIIFImage(Image):
    format: str = "image/jpeg"
    # as other services are defined, add them as options.
    service: list[ImageService | ExternalAuthService] = []


class Annotation(BaseID):
    context: str = Field(
        default="http://iiif.io/api/presentation/3/context.json", alias="@context"
    )
    type: str = "Annotation"
    motivation: str = "painting"
    target: AnyUrl
    body: IIIFImage | Choice | None = None

    def add_image(self, image: IIIFImage):
        if self.body is None:
            self.body = image
        elif isinstance(self.body, Choice):
            self.body.items.append(image)
        else:
            self.body = Choice(items=[self.body, image])


class AnnotationPage(BaseID):
    context: str = Field(
        default="http://iiif.io/api/presentation/3/context.json", alias="@context"
    )
    type: str = "AnnotationPage"
    items: list[Annotation] = []

    def add_annotation(self, annotation: Annotation):
        self.items.append(annotation)


class Canvas(BaseID):
    type: str = "Canvas"
    label: LanguageString
    height: int
    width: int
    behavior: list[str] | None  # ['paged'], ['facing-pages'], ['non-paged']
    items: list = []

    def add_annotation_page(self, annotation_page: AnnotationPage):
        self.items.append(annotation_page)


# Range should be defined here, but the collections site does not yet support it.
# Use a range for creating a table of contents within the client (Mirador viewer).
# For example, printed books could be broken into chapters, and manuscripts could
# be devided into books or major sections since many manuscripts have non-biblical texts


class Manifest(BaseID):
    context: str = Field(
        default="http://iiif.io/api/presentation/3/context.json", alias="@context"
    )
    type: str = "Manifest"
    label: LanguageString
    metadata: list[LabelValue] = []
    summary: LanguageString | None
    thumbnail: Thumbnail | None
    viewingDirection: str = "left-to-right"
    behavior: list[str] = ["paged"]
    rights: AnyUrl | str | None
    requiredStatement: LabelValue | None
    provider: list[Provider] | None
    homepage: Homepage | None
    partOf: PartOf | None
    items: list = []

    def add_canvas(self, canvas: Canvas):
        self.items.append(canvas)


class Collection(BaseID):
    context: str = Field(
        default="http://iiif.io/api/presentation/3/context.json", alias="@context"
    )
    type: str = "Collection"
    label: LanguageString
    requiredStatement: LabelValue | None
    items: list = []

    def add_manifest(self, manifest: Manifest):
        self.items.append(manifest)



from multiprocessing import context
from pydantic import BaseModel, AnyUrl, root_validator, validator, Field


class BaseID(BaseModel):

    @validator("id", pre=True)
    def slugify(s):
        if isinstance(s, str):
            return s.replace(" ", "%20")
        return s

    id: AnyUrl


class LanguageString(BaseModel):
    """A language string is a dictionary with language codes as keys and
    strings as values. The language codes are ISO 2-character codes.
    Add additional languages as needed, but once more than one exists,
    logic is needed to enable all to be optional, but one must be declared."""

    @root_validator(pre=True)
    def check_language_string(cls, values):
        if len(values) == 0:
            raise ValueError("At least one language string must be provided.")
        return values

    en: list[str] | None
    fr: list[str] | None
    de: list[str] | None
    it: list[str] | None
    es: list[str] | None


class LabelValue(BaseModel):
    """Used as the value of the requiredStatement and metadata items."""

    label: LanguageString
    value: LanguageString


class Image(BaseID):
    type: str = "Image"
    width: int | None
    height: int | None


class Thumbnail(Image):
    format: str = "image/jpeg"


class Logo(Image):
    format: str = "image/png"


class Homepage(BaseID):
    type: str = "Text"
    label: LanguageString
    format: str = "text/html"
    language: list[str]


class PartOf(BaseID):
    type: str = "Collection"


class Provider(BaseID):
    type: str = "Agent"
    label: LanguageString
    homepage: list[Homepage] | None
    logo: list[Logo] | None


class Choice(BaseModel):
    type: str = "Choice"
    items: list = []


class ImageService(BaseID):
    type: str = "ImageService3"
    profile: str = "level0"


class ExternalAuthService(BaseModel):
    context: str = Field(
        default="http://iiif.io/api/auth/1/context.json", alias="@context"
    )
    profile: str = "http://iiif.io/api/auth/1/external"
    label: LanguageString
    failureHeader: LanguageString
    failureDescription: LanguageString
